@@ -3,9 +3,11 @@ pub use crate::{
     builder::*,
     channel::*,
     component::*,
-    futures::{self, *},
+    event::{DomEvent, Eventable},
+    futures::*,
     model::*,
     patch::*,
+    relay::*,
     target::*,
     utils,
     view::*,
